#include <main.h>

//reads density and gate pots and if settings are changed
//sends data after 0.5 secs of settling

//9/5/21  flipped gate and density inputs per board layout
// added quiescent point as a user configurable parameter

//9/25/21  added serial commands and multiple bands

#rom 0x1F00 = {123} //default q point
#rom 0x1F01 = {0}  //stored settings flag 0=no stored data
//#rom 0x1F02 = {160,3,32,80,180,2,40,120,200,5,80,170,190,7,100,250}

// debug mode
//#define DEBUG 
//#define TABLEDUMP 

//#define CHECK_DATA 0
//#define SEND_DATA 1
//#define SEND_QPOINT 2
//#define SEND_DENS 3
//#define SEND_GATE 4
//#define SEND_AGC 5
//#define SEND_DONE 99

char band[5] = "LMPH";
char var[5] = "DGAQ";

unsigned int sendstate;


signed int8 dadc; //recent density ADC
signed int8 gadc; //recent gate ADC
signed int8 aadc; //recent agc adc
signed int8 olddadc; //previous density setting at last read
signed int8 oldgadc; //previous gate setting at last read
signed int8 oldaadc; //previous agc setting at last read
//unsigned int16 density;
//unsigned int16 gate;
int8 oflows;
int1 potmoved;             //one of the pots moved
int1 dpotmoved;            //dens pot moved
int1 gpotmoved;            //gate pot moved
int1 apotmoved;            //agc pot moved
int8 rxbuf[8];             //UART array
int8 rxchar;               //UART data
int1 data_ok;              //UART data valid
int8 cptr;                 //UART array pointer
int16 datain;              //incoming numeric value from serial
int8 setting;              // numeric value of final setting from serial command

int8 agc;
int8 gtime;
int8 qpoint;


int8 x=0;
int8 ii;
//int8 jj;
int8 memok;
int8 mem[20];
int8 memptr;
int8 memb;
//int8 memp;  //memory location + memp

//mem map
// 0 = low density
// 1 = low gate
// 2 = low agc
// 3 = low gtime
// 4 = low quiescent
// 5 = mid density
// 6 = mid gate
// 7 = mid agc
// 8 = mid gtime
// 9 = mid quiescent
// 10 = pres density
// 11 = pres gate
// 12 = pres agc
// 13 = pres gtime
// 14 = pres quiescent
// 15 = high density
// 16 = high gate
// 17 = high agc
// 18 = high gtime
// 19 = high quiescent



int8 CONST gconst[256] = {188,188,188,188,188,188,188,188,188,188,188,188,188,188,
188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,188,183,
177,172,167,163,158,154,151,147,143,140,137,134,131,128,125,123,120,118,116,113,
111,109,107,105,104,102,100,99,97,95,94,92,91,90,88,87,86,85,83,82,81,80,79,78,
77,76,75,74,73,72,71,71,70,69,68,67,67,66,65,64,64,63,62,62,61,61,60,59,59,58,58,
57,56,56,55,55,54,54,53,53,52,52,52,51,51,50,50,49,49,49,48,48,47,47,47,47,47,47,
47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,
47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47};


#INT_RDA //Serial port interrupt circular buffer
void  RDA_isr(void) 
{
   cptr++;
   if (cptr>8) {cptr=0;}              //reset for excess chars
   rxchar=getc(); 
         
   if ( rxchar == '#' ) {               //command start flag '#'
   memset(rxbuf,0,sizeof(rxbuf));      //clear buffer
   data_ok=0;                          //clear data_ok flag
   cptr=0;                             //set pointer to zero
   rxbuf[cptr] = rxchar;}              //store '#' in 0
   else if (rxchar == 13)              //command end flag CR
   { rxbuf[cptr]=rxchar;               //store CR in last ptr 
    data_ok=1;}
   else { rxbuf[cptr]=rxchar;}         //store everything else
      
}

#INT_TIMER0 //interrupt every 4ms
void  TIMER0_isr(void) 
{



if (abs(dadc - olddadc) > 10) 
{oflows=0;
dpotmoved=1;
olddadc = dadc;}


if (abs(gadc - oldgadc) > 10) 
{oflows=0;
gpotmoved=1;
oldgadc = gadc;}

if (abs(aadc - oldaadc) > 10) 
{oflows=0;
apotmoved=1;
oldaadc = aadc;}

if ((dpotmoved==1) | (gpotmoved==1) | (apotmoved==1)) 
{oflows++;
   if (oflows > 64) 
   {oflows=0;
   potmoved=1;
   }
}

}






void handlecoms()
{


#if defined(DEBUG)
printf("#");
printf("%c",rxbuf[1]);
printf("%c",rxbuf[2]);
printf("%u",setting);
if (rxbuf[2]=='A') {printf(" %u ",gconst[agc]);}
printf("%c",13);
#else
putc('#');
putc(rxbuf[1]); //band
putc(rxbuf[2]); //parameter type
putc(setting);
if (rxbuf[2]=='A') {putc(gconst[agc]);} //lookup gtime only for agc
putc(13);
#endif

//store settings to array
memb=0;
if ((rxbuf[1]=='L') | (rxbuf[1]=='M') | (rxbuf[1]=='P') | (rxbuf[1]=='H'))
{
if ((rxbuf[1]=='L') ) {memb=0;}
if ((rxbuf[1]=='M') ) {memb=4;}
if ((rxbuf[1]=='P') ) {memb=8;}
if ((rxbuf[1]=='H') ) {memb=12;}
if (rxbuf[2]=='D') {mem[memb]=setting;}
if (rxbuf[2]=='G') {mem[memb+1]=setting;}
if (rxbuf[2]=='A') {mem[memb+2]=setting;}
if (rxbuf[2]=='Q') {mem[memb+3]=setting;}
}
else if (rxbuf[1]=='A')
{
   if (rxbuf[2]=='D') 
   {mem[0]=setting;
    mem[4]=setting;
    mem[8]=setting;
    mem[12]=setting;}
   if (rxbuf[2]=='G') 
   {mem[1]=setting;
    mem[5]=setting;
    mem[9]=setting;
    mem[13]=setting;}
   if (rxbuf[2]=='A') 
   {mem[2]=setting;
    mem[6]=setting;
    mem[10]=setting;
    mem[14]=setting;}
   if (rxbuf[2]=='Q') 
   {mem[3]=setting;
    mem[7]=setting;
    mem[11]=setting;
    mem[15]=setting;}
}
memb=0;
#if defined(TABLEDUMP)
for(x=0;x<16;x++) 
{printf("mem %u \r",mem[x]);}
#endif

}


void processdata()
{
//printf("sendstate = %u \r",sendstate);
         //clear data ok flag for next command
         data_ok=0;
         //verify if command is specific to a band
         if ( (rxbuf[1] == 'L') | (rxbuf[1] == 'M') | (rxbuf[1] == 'P') | (rxbuf[1] == 'H') | (rxbuf[1] == 'A')) 
         {  
            //verify if the command is a parameter setting
            if ( (rxbuf[2] == 'D') | (rxbuf[2] == 'G') | (rxbuf[2] == 'A') | (rxbuf[2] == 'Q'))
            {
            
               
               if (!cptr==0) {
               //if three digits and upper digit is <3 
               if ((cptr == 6) && (rxbuf[6] == 13)) { datain = (((unsigned int16)(rxbuf[3]-48)*100)+((unsigned int16)(rxbuf[4]-48)*10)+(unsigned int16)(rxbuf[5]-48));}
               if ((cptr == 5) && (rxbuf[5] == 13)) { datain = (((unsigned int16)(rxbuf[3]-48)*10)+(unsigned int16)(rxbuf[4]-48));}
               if ((cptr == 4) && (rxbuf[4] == 13)) { datain = ((unsigned int16)(rxbuf[3]-48));}
               setting = datain;
               }
                  if ( setting >= 0 && setting <= 255 )  
                  { 
                     if ( rxbuf[2] == 'D' ) {handlecoms(); }
                     if ( rxbuf[2] == 'G' ) {handlecoms(); }
                     if ( rxbuf[2] == 'A' ) 
                     {gtime = gconst[agc];
                     handlecoms(); }
                     if ( rxbuf[2] == 'Q' ) 
                     { //jj = (255-datain);
                     //qpoint = (((170) * (unsigned int16)jj)/255) + (80);
                     write_program_eeprom(0x1F00,setting);
                     //setting=qpoint;
                     handlecoms(); }
                  }
                  else{
                  memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
                  data_ok=0;  //clear data ok flag
                  cptr=0;}
               
            }
            else
            memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
            data_ok=0; 
            cptr=0;}
            
            //store current array data to eeprom
            if ((rxbuf[0] == '#') && (rxbuf[1] == 'S') && ( rxbuf[2] == 'D') && (rxbuf[3] == 13))
            {
            for(x=0;x<16;x++) {
            write_program_eeprom((0x1F02 + x),mem[x]); 
            write_program_eeprom(0x1F01,50);
            memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
            data_ok=0;
            }
            }
            //clear eeprom data and clear data flag
           
            if ((rxbuf[0] == '#') && (rxbuf[1] == 'C') && ( rxbuf[2] == 'D') && (rxbuf[3] == 13))
            {
            for(x=0;x<16;x++) {
            write_program_eeprom((0x1F02 + x),0); 
            write_program_eeprom(0x1F01,0);
            write_program_eeprom(0x1F00,123); //reset qpoint to midscale
            memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
            data_ok=0;
            sendstate=0; //reset q point and return to panel control
            }
            }
            
            //read out all settings and send to band PICS
            if ((rxbuf[0] == '#') && (rxbuf[1] == 'R') && ( rxbuf[2] == 'D') && (rxbuf[3] == 13))
            {
            memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
            int8 a;
            int8 b;
            int8 ctr=0;
            int8 tempdata;
            //char band[5] = "LMPH";
            //char var[5] = "DGAQ";
            for (a=0;a<4;a++) {
               for (b=0;b<4;b++) {
               rxbuf[1] = band[a];
               rxbuf[2] = var[b];
               tempdata = read_program_eeprom(0x1F02 + ctr);
               setting = make8(tempdata,0);
               handlecoms();
               //printf("%c %c %u \r",band[a],var[b],setting);
               ctr+=1;
               delay_ms(50);
               }
            }
            
            }
            //software query of all current parameters
            if ((rxbuf[0] == '#') && (rxbuf[1] == 'S') && ( rxbuf[2] == 'Q') && (rxbuf[3] == 13))
            {
            memset(rxbuf,0,sizeof(rxbuf)); //clear the recieve buffer after sending
            int8 c;
            for(c=0;c<16;c++) 
            {if (c < 15) {printf("%u,",mem[c]);}
            else {printf("%u\r\n",mem[c]);}
            }
            }
            
            
            
}





void main()
{
   
   
   
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_128|RTCC_8_bit);      //4ms overflow
   
   
   setup_adc_ports(sAN0|sAN1|sAN2);
   setup_adc_ports(VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL);
   
   
   //jj = make8(read_program_eeprom(0x1F7D),0);
   //if ( jj == 'L') {printf("got L \r");}
   
         
 
   enable_interrupts(INT_RDA);
   //enable_interrupts(INT_TIMER0);
   enable_interrupts(GLOBAL);
   
   // do initial setting read
      set_adc_channel(0);
      delay_us(50);
      gadc = read_adc();
      oldgadc = gadc;
      set_adc_channel(1); 
      delay_us(50);
      dadc = read_adc();
      olddadc = dadc;
      set_adc_channel(2); 
      delay_us(50);
      aadc = read_adc();
      oldaadc = aadc;
   
   dpotmoved=0; //send density
   gpotmoved=0; //send gate
   apotmoved=0; //send agc
   potmoved=0;  //send all
   sendstate = 0;
   memptr=0;
   cptr=0;
   
   delay_ms(1000);

   while(TRUE)
   {
      
      set_adc_channel(0);
      delay_us(50);
      gadc = read_adc(); //gate pot
      set_adc_channel(1); 
      delay_us(50);
      dadc = read_adc(); //dens pot
      set_adc_channel(2); 
      delay_us(50);
      aadc = read_adc(); //agc pot
      
      // 200 - 50 is the range (150)
      int8 pdensity =  (((150) * (unsigned int16)dadc)/255) + (50); 
      // 40 - 1 is the range (39)
      int8 pgate = (((39) * (unsigned int16)gadc)/255) + (1);
      // 32 - 128 is the range  32+96
      int8 pagc = (((96) * (unsigned int16)aadc)/255) + (32);
     
     
     //powerup send settings once
     while( sendstate != 99 )
     {
     
     
     switch(sendstate) 
     {
         case 0:
            memok = read_program_eeprom(0x1F01);
            if (memok == 0) {  sendstate = 2; }
            else {sendstate = 1; }
            break;
         case 1:
            disable_interrupts(INT_TIMER0);
            rxbuf[0]='#';
            rxbuf[1]='R';
            rxbuf[2]='D';
            rxbuf[3]=13;
            data_ok=1;
            cptr=0;
            processdata();
            sendstate = 99;
            break;
         case 2:
               ii = read_program_eeprom(0x1F00);
               if (!(ii == 123))  //programmed initially at 123
               { qpoint = make8(ii,0);}
               else { qpoint = 123;}
               setting=qpoint;
               rxbuf[1]='A';
               rxbuf[2]='Q';
               data_ok=1;
               cptr=0;
               processdata();
               sendstate = 3;
               break;
         case 3:
               setting=pdensity;
               rxbuf[1]='A';
               rxbuf[2]='D';
               data_ok=1;
               cptr=0;
               processdata();
               sendstate = 4;
               break;
         case 4:
               setting=pgate;
               rxbuf[1]='A';
               rxbuf[2]='G';
               data_ok=1;
               cptr=0;
               processdata();
               sendstate = 5;
               break;
         case 5:
               setting=pagc;
               rxbuf[1]='A';
               rxbuf[2]='A';
               data_ok=1;
               cptr=0;
               processdata();
               sendstate = 99;
               potmoved=0;
               enable_interrupts(INT_TIMER0);
               break;
         
               
     }
     
     }
     
    // update on pot changes
     if ((potmoved == 1) && (sendstate == 99))  //pot change detected
     {
         potmoved=0;
         if (dpotmoved == 1)
         {  dpotmoved=0;
            setting=pdensity;
            rxbuf[1]='A';
            rxbuf[2]='D';
            data_ok=1;
            cptr=0;
            processdata();
         }
         if (gpotmoved == 1)
         {  gpotmoved=0;
            setting=pgate;
            rxbuf[1]='A';
            rxbuf[2]='G';
            data_ok=1;
            cptr=0;
            processdata();
         }
         if (apotmoved == 1)
         {  apotmoved=0;
            setting=pagc;
            rxbuf[1]='A';
            rxbuf[2]='A';
            data_ok=1;
            cptr=0;
            processdata();
         } 
     potmoved=0; //if none then clear all
     }
     
      //data came in from the serial port
      
      if (data_ok == 1)  //received a # char and CR
      {
         processdata(); 
      } //end of data ok
      
} //while loop
         
} //end of main
      
      
   



